# Autogenerated constants for Flex service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_FLEX = const(0x1f47c6c6)


class FlexVariant(IntEnum):
    LINEAR22INCH = const(0x1)
    LINEAR45INCH = const(0x2)


JD_FLEX_REG_BENDING = const(JD_REG_READING)
JD_FLEX_REG_VARIANT = const(JD_REG_VARIANT)
JD_FLEX_PACK_FORMATS = {
    JD_FLEX_REG_BENDING: "u0.16",
    JD_FLEX_REG_VARIANT: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client, EventHandlerFn, UnsubscribeFn
from .constants import *
from typing import Optional


class RelayClient(Client):
    """
    A switching relay.
    Implements a client for the `Relay <https://microsoft.github.io/jacdac-docs/services/relay>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_RELAY, JD_RELAY_PACK_FORMATS, role)


    @property
    def closed(self) -> Optional[bool]:
        """
        Indicates whether the relay circuit is currently energized (closed) or not., 
        """
        return self.register(JD_RELAY_REG_CLOSED).bool_value()

    @closed.setter
    def closed(self, value: bool) -> None:
        self.register(JD_RELAY_REG_CLOSED).set_values(value)


    @property
    def variant(self) -> Optional[RelayVariant]:
        """
        (Optional) Describes the type of relay used., 
        """
        return self.register(JD_RELAY_REG_VARIANT).value()

    @property
    def max_switching_current(self) -> Optional[int]:
        """
        (Optional) Maximum switching current for a resistive load., _: mA
        """
        return self.register(JD_RELAY_REG_MAX_SWITCHING_CURRENT).value()

    def on_active(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when relay goes from `inactive` to `active` state.
        Normally open (NO) relays close the circuit when activated.
        """
        return self.on_event(JD_RELAY_EV_ACTIVE, handler)

    def on_inactive(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Emitted when relay goes from `active` to `inactive` state.
        Normally closed (NC) relays open the circuit when activated.
        """
        return self.on_event(JD_RELAY_EV_INACTIVE, handler)

    



from typing import Any
from ..bus import Bus
from .client import RelayClient
from time import sleep

if __name__ == '__main__':
    bus = Bus()

    def active(data: Any):
        print("active")

    def inactive(data: Any):
        print("inactive")

    rel = RelayClient(bus, "relay")
    rel.on_active(active)
    rel.on_inactive(inactive)

    while True:
        rel.closed = False
        sleep(1)
        print("open: ", rel.closed)
        rel.closed = True
        sleep(1)
        print("closed: ", rel.closed)



# Autogenerated constants for Sound level service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_SOUND_LEVEL = const(0x14ad1a5d)
JD_SOUND_LEVEL_REG_SOUND_LEVEL = const(JD_REG_READING)
JD_SOUND_LEVEL_REG_ENABLED = const(JD_REG_INTENSITY)
JD_SOUND_LEVEL_REG_MIN_DECIBELS = const(0x81)
JD_SOUND_LEVEL_REG_MAX_DECIBELS = const(0x82)
JD_SOUND_LEVEL_REG_LOUD_THRESHOLD = const(JD_REG_ACTIVE_THRESHOLD)
JD_SOUND_LEVEL_REG_QUIET_THRESHOLD = const(JD_REG_INACTIVE_THRESHOLD)
JD_SOUND_LEVEL_EV_LOUD = const(JD_EV_ACTIVE)
JD_SOUND_LEVEL_EV_QUIET = const(JD_EV_INACTIVE)
JD_SOUND_LEVEL_PACK_FORMATS = {
    JD_SOUND_LEVEL_REG_SOUND_LEVEL: "u0.16",
    JD_SOUND_LEVEL_REG_ENABLED: "u8",
    JD_SOUND_LEVEL_REG_MIN_DECIBELS: "i16",
    JD_SOUND_LEVEL_REG_MAX_DECIBELS: "i16",
    JD_SOUND_LEVEL_REG_LOUD_THRESHOLD: "u0.16",
    JD_SOUND_LEVEL_REG_QUIET_THRESHOLD: "u0.16"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client
from .constants import *
from typing import Optional


class VibrationMotorClient(Client):
    """
    A vibration motor.
    Implements a client for the `Vibration motor <https://microsoft.github.io/jacdac-docs/services/vibration>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_VIBRATION_MOTOR, JD_VIBRATION_MOTOR_PACK_FORMATS, role)


    @property
    def enabled(self) -> Optional[bool]:
        """
        Determines if the vibration motor responds to vibrate commands., 
        """
        return self.register(JD_VIBRATION_MOTOR_REG_ENABLED).bool_value()

    @enabled.setter
    def enabled(self, value: bool) -> None:
        self.register(JD_VIBRATION_MOTOR_REG_ENABLED).set_values(value)


    



from ..bus import Bus
from .client import ThermometerClient
from time import sleep

if __name__ == '__main__':
    bus = Bus()

    client = ThermometerClient(bus, "thermometer")
    while True:
        print("temp: ", client.temperature, "error:", client.temperature_error)
        sleep(0.2)



from ..bus import Bus
from .client import BarometerClient
from time import sleep

if __name__ == '__main__':
    bus = Bus()

    barometer = BarometerClient(bus, "barometer")
    while True:
        print("pressure: ", barometer.pressure, "e:", barometer.pressure_error)
        sleep(1)



# Autogenerated constants for Thermocouple service
from enum import IntEnum
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_THERMOCOUPLE = const(0x143ac061)


class ThermocoupleVariant(IntEnum):
    TYPE_K = const(0x1)
    TYPE_J = const(0x2)
    TYPE_T = const(0x3)
    TYPE_E = const(0x4)
    TYPE_N = const(0x5)
    TYPE_S = const(0x6)
    TYPE_R = const(0x7)
    TYPE_B = const(0x8)


JD_THERMOCOUPLE_REG_TEMPERATURE = const(JD_REG_READING)
JD_THERMOCOUPLE_REG_MIN_TEMPERATURE = const(JD_REG_MIN_READING)
JD_THERMOCOUPLE_REG_MAX_TEMPERATURE = const(JD_REG_MAX_READING)
JD_THERMOCOUPLE_REG_TEMPERATURE_ERROR = const(JD_REG_READING_ERROR)
JD_THERMOCOUPLE_REG_VARIANT = const(JD_REG_VARIANT)
JD_THERMOCOUPLE_PACK_FORMATS = {
    JD_THERMOCOUPLE_REG_TEMPERATURE: "i22.10",
    JD_THERMOCOUPLE_REG_MIN_TEMPERATURE: "i22.10",
    JD_THERMOCOUPLE_REG_MAX_TEMPERATURE: "i22.10",
    JD_THERMOCOUPLE_REG_TEMPERATURE_ERROR: "u22.10",
    JD_THERMOCOUPLE_REG_VARIANT: "u8"
}



# Autogenerated file. Do not edit.
from jacdac.bus import Bus, Client, EventHandlerFn, UnsubscribeFn
from .constants import *
from typing import Optional


class HidAdapterClient(Client):
    """
    A service for configuring how Jacdac device map to HID input events. Users can have multiple configurations and swap between them by writing to `current_configuration`.
    Implements a client for the `HID Adapter <https://microsoft.github.io/jacdac-docs/services/hidadapter>`_ service.

    """

    def __init__(self, bus: Bus, role: str) -> None:
        super().__init__(bus, JD_SERVICE_CLASS_HID_ADAPTER, JD_HID_ADAPTER_PACK_FORMATS, role)


    @property
    def num_configurations(self) -> Optional[int]:
        """
        The number of configurations stored on the server., 
        """
        return self.register(JD_HID_ADAPTER_REG_NUM_CONFIGURATIONS).value()

    @num_configurations.setter
    def num_configurations(self, value: int) -> None:
        self.register(JD_HID_ADAPTER_REG_NUM_CONFIGURATIONS).set_values(value)


    @property
    def current_configuration(self) -> Optional[int]:
        """
        The current configuration the server is using., 
        """
        return self.register(JD_HID_ADAPTER_REG_CURRENT_CONFIGURATION).value()

    @current_configuration.setter
    def current_configuration(self, value: int) -> None:
        self.register(JD_HID_ADAPTER_REG_CURRENT_CONFIGURATION).set_values(value)


    def on_changed(self, handler: EventHandlerFn) -> UnsubscribeFn:
        """
        Event that notifies clients that the server has swapped to a new configuration or changed key bindings.
        """
        return self.on_event(JD_HID_ADAPTER_EV_CHANGED, handler)


    def set_binding(self, configuration_number: int, binding_index: int, device_id: int, service_class: int, trigger_value: int, trigger_context: int, service_index: int, selector: int, modifiers: int) -> None:
        """
        Stores the given binding on the server. If a binding exists at this index, the new binding will replace it.
        """
        self.send_cmd_packed(JD_HID_ADAPTER_CMD_SET_BINDING, configuration_number, binding_index, device_id, service_class, trigger_value, trigger_context, service_index, selector, modifiers)

    def clear_binding(self, configuration_number: int, binding_index: int) -> None:
        """
        Clears a specific binding stored on the device.
        """
        self.send_cmd_packed(JD_HID_ADAPTER_CMD_CLEAR_BINDING, configuration_number, binding_index)

    def clear_configuration(self, configuration_number: int) -> None:
        """
        Clears a specific configuration stored on the device.
        """
        self.send_cmd_packed(JD_HID_ADAPTER_CMD_CLEAR_CONFIGURATION, configuration_number)

    def clear(self, ) -> None:
        """
        Clears all configurations and bindings stored on the device.
        """
        self.send_cmd_packed(JD_HID_ADAPTER_CMD_CLEAR, )
    




from .bus import Bus
from configparser import ConfigParser
from jacdac.logger.constants import JD_LOGGER_PRIORITY_SILENT


def create_bus_from_config() -> Bus:
    """Creates a bus from configuration files

    Raises:
        RuntimeError: no jacdac section was found

    Returns:
        Bus: A Jacdac bus
    """

    config = ConfigParser()
    config.read(["./jacdac.ini", "./.jacdac/config.ini", "./setup.cfg"])
    if not "jacdac" in config:
        raise RuntimeError("no jacdac session found in configuration files")

    cfg = config["jacdac"]
    return Bus(device_id=cfg.get("device_id", None),
               product_identifier=cfg.getint(
        "product_identifier", None),
        firmware_version=cfg.get(
        "firmware_version", None),
        device_description=cfg.get(
        "device_description", None),
        disable_logger=cfg.getboolean(
        "disable_logger", False),
        disable_role_manager=cfg.getboolean(
        "disable_role_manager", False),
        disable_brain=cfg.getboolean(
        "disable_brain", False),
        disable_dev_tools=cfg.getboolean(
        "disable_dev_tools", False),
        hf2_portname=cfg.get("hf2_portname", None),
        default_logger_min_priority=cfg.getint(
        "default_logger_min_priority", JD_LOGGER_PRIORITY_SILENT),
        role_manager_file_name=cfg.get(
        "role_manager_file_name", None),
        settings_file_name=cfg.get(
        "settings_file_name", None)
    )



# Autogenerated constants for HID Adapter service
from jacdac.constants import *
from jacdac.system.constants import *
JD_SERVICE_CLASS_HID_ADAPTER = const(0x1e5758b5)
JD_HID_ADAPTER_REG_NUM_CONFIGURATIONS = const(0x80)
JD_HID_ADAPTER_REG_CURRENT_CONFIGURATION = const(0x81)
JD_HID_ADAPTER_CMD_GET_CONFIGURATION = const(0x80)
JD_HID_ADAPTER_CMD_SET_BINDING = const(0x82)
JD_HID_ADAPTER_CMD_CLEAR_BINDING = const(0x83)
JD_HID_ADAPTER_CMD_CLEAR_CONFIGURATION = const(0x84)
JD_HID_ADAPTER_CMD_CLEAR = const(0x85)
JD_HID_ADAPTER_EV_CHANGED = const(JD_EV_CHANGE)
JD_HID_ADAPTER_PACK_FORMATS = {
    JD_HID_ADAPTER_REG_NUM_CONFIGURATIONS: "u8",
    JD_HID_ADAPTER_REG_CURRENT_CONFIGURATION: "u8",
    JD_HID_ADAPTER_CMD_GET_CONFIGURATION: "b[12] u8",
    JD_HID_ADAPTER_CMD_SET_BINDING: "u8 u8 b[2] u64 u32 u32 u8 u8 u16 u16",
    JD_HID_ADAPTER_CMD_CLEAR_BINDING: "u8 u8",
    JD_HID_ADAPTER_CMD_CLEAR_CONFIGURATION: "u8"
}
